/* -*- Mode: C++; tab-width: 2; indent-tabs-mode:nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
#include <signal.h>
#include <stdlib.h>
#include <unistd.h>
#include <execinfo.h>

#include "nsCrashRepService.h"
#include "nsIObserverService.h"

#include "nsISupportsArray.h"
#include "nsXPCOM.h"
#include "nsISupportsPrimitives.h"
#include "nsIServiceManager.h"
#include "nsIDOMWindowInternal.h"
#include "nsDependentString.h"

#include "nsDirectoryService.h"
#include "nsDirectoryServiceDefs.h"

NS_INTERFACE_MAP_BEGIN(nsCrashRepService)
  NS_INTERFACE_MAP_ENTRY(nsICrashRepService)
  NS_INTERFACE_MAP_ENTRY(nsIObserver)
  NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsICrashRepService)
NS_INTERFACE_MAP_END_THREADSAFE

NS_IMPL_ADDREF(nsCrashRepService)
NS_IMPL_RELEASE(nsCrashRepService)

nsCrashRepService* nsCrashRepService::sInstance = nsnull;

void ah_crap_handler(int signum)
{
  int i;
  void *bt[128];
  char signal[3];
  int bt_size;

  FILE *f;
  char line[200];
  char lines[100000]; // max 500 lines with 200 chars
  char bts[12800]; // every bt can have 100 chars
  line[0]='\0';
  lines[0]='\0';
  bts[0]='\0';
  printf("Sorry our application crashed with error %d\n", signum);
  bt_size = backtrace(bt, sizeof(bt) / sizeof(void *));
  for (i=0; i<bt_size; i++)
  {
    sprintf(bts,"%s%p\n",bts,bt[i]);
  }
  f = fopen("/proc/self/maps","r");
  if (f)
  {
    int i = 0;
    while((fgets(line, sizeof(line)-1, f) != NULL) && (i<500))
    {
      sprintf(lines,"%s%s\n",lines,line);
      i++;
    }
    fclose(f);
  }
  printf("DONE\n");
  nsCOMPtr<nsIFile> appFileSpec;
  nsresult rv;
  nsCAutoString path;
  rv = NS_GetSpecialDirectory( NS_XPCOM_CURRENT_PROCESS_DIR, getter_AddRefs(appFileSpec));
//  rv = NS_GetSpecialDirectory( NS_XPCOM_COMPONENT_DIR, getter_AddRefs(iconFileSpec));
  rv = appFileSpec->GetNativePath(path);
//  path.Append("/crashrep");
  chdir(path.get());
  path.Append("/crashrep");
  sprintf(signal, "%2d", signum);
  i = execl(path.get(), "crashrep", (const char*)bts, (const char*)lines, (const char*)signal, (char *)0);
  exit(-1);
}

nsCrashRepService::nsCrashRepService()
{
  fprintf(stderr,"Crash Reporting Service startup\n");
  signal(SIGFPE, ah_crap_handler);
  signal(SIGSEGV, ah_crap_handler);
  signal(SIGILL, ah_crap_handler);
  signal(SIGABRT, ah_crap_handler);
}

nsCrashRepService::~nsCrashRepService()
{
}

nsresult
nsCrashRepService::Init()
{
  return NS_OK;
}

nsCrashRepService
*nsCrashRepService::GetInstance()
{
  if (!sInstance) {
    sInstance = new nsCrashRepService();
    if (!sInstance)
      return nsnull;

    NS_ADDREF(sInstance);  // addref for sInstance global

    if (NS_FAILED(sInstance->Init())) {
      NS_RELEASE(sInstance);

      return nsnull;
    }
  }

  NS_ADDREF(sInstance);   // addref for the getter

  return sInstance;
}

void
nsCrashRepService::ReleaseInstance()
{
  NS_IF_RELEASE(sInstance);
}


// ----------- nsIObserver Methods (1) -------------------

NS_IMETHODIMP
nsCrashRepService::Observe(nsISupports *aSubject, const char *aTopic,
                         const PRUnichar *aData)
{
  return NS_OK;
}
